@@ -5,33 +5,80 @@ use warnings;
 
 use Carp qw(croak);
 use Symbol ();
-use Tie::Hash ();
 
-our ($VERSION, @ISA, $DU_BIN);
+our ($VERSION, $DU_BIN);
 
-$VERSION = '0.21';
-@ISA = qw(Tie::StdHash);
+$VERSION = '0.22';
 
 $DU_BIN = '/usr/bin/du';
 
 sub TIEHASH
 {
     my $class = shift;
-    return bless _tie(@_), $class;
-}
-sub UNTIE {}
 
-sub _tie
-{
     my $du = _locate_du();
     my $path = shift @_;
     my @opts = @_;
 
     _validate($path, \@opts);
 
-    return _parse_usage($du, $path, @opts);
+    return bless { du => $du, path => $path, opts => \@opts }, $class;
 }
 
+sub EXISTS
+{
+    my $self = shift;
+    my ($key) = @_;
+
+    my $usage = _parse_usage($self->{du}, $self->{path}, $self->{opts}, $key);
+
+    return exists $usage->{$key};
+}
+
+sub FETCH
+{
+    my $self = shift;
+    my ($key) = @_;
+
+    my $usage = _parse_usage($self->{du}, $self->{path}, $self->{opts}, $key);
+
+    return $usage->{$key};
+}
+
+sub FIRSTKEY
+{
+    my $self = shift;
+
+    my $usage = _parse_usage($self->{du}, $self->{path}, $self->{opts}, undef);
+
+    my @keys = sort keys %$usage;
+    my $key = shift @keys;
+    delete $self->{'keys'};
+    $self->{'keys'} = [ @keys ];
+
+    return $key;
+}
+
+sub NEXTKEY
+{
+    my $self = shift;
+
+    return shift @{$self->{'keys'}};
+}
+
+sub SCALAR
+{
+    my $self = shift;
+
+    my $usage = _parse_usage($self->{du}, $self->{path}, $self->{opts}, undef);
+
+    return scalar %$usage;
+}
+
+sub UNTIE {}
+
+*CLEAR = *DELETE = *STORE = sub { croak 'Tied hash is read-only' };
+
 sub _validate
 {
     my ($path, $opts) = @_;
@@ -75,15 +122,17 @@ sub _locate_du
 
 sub _parse_usage
 {
-    my ($du, $path, @opts) = @_;
+    my ($du, $path, $opts, $key) = @_;
     $path ||= do { require Cwd; Cwd::getcwd() };
 
     my $pipe = Symbol::gensym();
-    open($pipe, "$du @opts $path |") or exit(1);
+    open($pipe, "$du @$opts $path |") or exit(1);
 
     my %usage;
     while (my $line = <$pipe>) {
         my ($size, $item) = $line =~ /^(.+?) \s+? (.+)$/x;
+        # optimisation for EXISTS/FETCH
+        next if defined $key && $key ne $item;
         $usage{$item} = $size;
     }
 
@@ -103,10 +152,23 @@ Tie::DiskUsage - Tie disk usage to a hash
 
  use Tie::DiskUsage;
 
+ # common use
  tie %usage, 'Tie::DiskUsage', '/var', '-h';
  print $usage{'/var/log'};
  untie %usage;
 
+ # also implemented
+ exists $usage{'/var/log'};
+ keys   %usage;
+ values %usage;
+ each   %usage;
+ scalar %usage;
+
+ # fatal error
+ %usage = ();
+ delete $usage{'/var/log'};
+ $usage{'/var/log'} = 1024 ** 2;
+
 =head1 DESCRIPTION
 
 C<Tie::DiskUsage> ties the disk usage, which is extracted from the output
@@ -127,6 +189,9 @@ automatic search for C<du>).
 Processing output of C<du(1)> requires that each output line is ended
 by a newline.
 
+In order to successfully run C<du(1)> on a given path, you might need to
+elevate permissions of the process using C<Tie::DiskUsage>.
+
 =head1 SEE ALSO
 
 L<perlfunc/tie>, du(1), L<Filesys::DiskUsage>, L<Sys::Statistics::Linux>
